"""
    Run! a game created for Pyweek
    Copyright (C) <2010> Thomas Berglund

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import pyglet
from pyglet.gl import *

class GameObject(object):
	def __init__(self, x, y, image, width, height, color=(1, 0.2, 1, 1), vx=0, vy=0, physical=False):
		self.x = x
		self.y = y
		self.width = width
		self.height = height
		if image != None:
			self.image = pyglet.resource.image(image)
			self.width, self.height = self.image.width, self.image.height
		else:
			self.image = image
		self.color = color
		self.vx = vx
		self.vy = vy
		self.fric = 0.9
		self.dead = False
		self.physical = physical
		
	def intersects(self, other):
		if self.x > other.x + other.width or \
		   self.x + self.width < other.x or \
		   self.y > other.y + other.height or \
		   self.y + self.height < other.y:
			return False
		return True
		
	def draw(self):
		glColor4f(*self.color)
		if self.image == None:
			pyglet.graphics.draw(4, GL_QUADS,
				('v2f', (self.x, self.y, 
						self.x + self.width, self.y,
						self.x + self.width, self.y + self.height,
						self.x, self.y + self.height))
			)
		else:
			self.image.blit(self.x, self.y)
		
	def move(self, dt):
		self.x += self.vx
		self.y += self.vy
		
	def collision(self, obj, world):
		if self.physical:
			if obj and world:
				self.colliding = True
				if self.y < obj.y + obj.height:
					self.y = obj.y + obj.height
			else:
				self.colliding = False