"""
    Run! a game created for Pyweek
    Copyright (C) <2010> Thomas Berglund

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""


import pyglet
from pyglet.window import key
from pyglet.gl import *

import random

from GameObject import *
from particles import *

pyglet.resource.path = ["data", "data/gfx", "data/sfx"]
pyglet.resource.reindex()

class Comet(Emitter):
	def __init__(self, x, y):
		self.x = random.randint(int(x)+400, int(x)+800)
		self.y = 600
		self.vx = 3
		self.vy = -20
		super(Comet, self).__init__(self.x, self.y, (0, 40), (0, 30), "cloud.png","comet.png",
					    relative=True, width=40, height=40, color=(1, 1, 1, 1), vx=4, vy=-20)
		self.max_particles = 20
		self.time_between_particles = 0.03
		self.time_since_last_particle = 0
		self.fric = 0

	def move(self, dt):
		self.time_since_last_particle += dt
		if len(self.particles) < self.max_particles and \
				self.time_since_last_particle > self.time_between_particles:
			self.create_particle(0, 0, 1, 2)
			self.time_since_last_particle = 0
		super(Comet, self).move(dt)
	
		
class FireWall(Emitter):
	def __init__(self):
		#def __init__(self, x, y, diff_x, diff_y, particle_image, emitter_image, width=0, height=0, color=(0, 0, 0, 0), vx=0, vy=0):
		super(FireWall, self).__init__(-600, -100, (0, 800), (0, 600), None, None,
						width=600, height=800, color=(1, 0.4, 0.2, 1), vx=11)
		#self.cloud_image = pyglet.image.load("../cloud.png")
		#self.fire_image = pyglet.image.load("../fire.png")
		
	def move(self, dt):
		#while len(self.particles) < self.max_particles:
		#	self.create_particle(10, 10, min_ttl=2, max_ttl=12, fade=True)
		
		super(FireWall, self).move(dt)
		
	def collide(self, obj):
		obj.dead = True

class Player(GameObject):
	def __init__(self):
		super(Player, self).__init__(380, 300, None, 40, 40)
		self.speed = 3
		self.jumping = False
		self.jump_timer = 0
		self.colliding = False
		self.keys = pyglet.window.key.KeyStateHandler()
		self.fric = 0.8
		self.physical = True
		self.zoom = False
		self.zoom_timer = 0
		self.zoom_reload = 0
		
	def move(self, dt):
		if self.keys[key.RIGHT]:
			self.vx += self.speed
		if self.keys[key.LEFT]:
			self.vx -= self.speed
		if self.keys[key.UP] and self.jumping == False and self.colliding:
			self.jumping = True
			self.jump_timer = 0.2
			self.keys[key.UP] = False
		if self.keys[key.Z] and self.zoom_reload <= 0:
			self.zoom = True
			self.zoom_reload = 5
			self.zoom_timer = 0.4
			self.color = (0, 0, 1, 1)
			
		if self.zoom and self.zoom_timer > 0:
			self.vx += 4
			self.zoom_timer -= dt
		else:
			self.zoom = False
			
		if self.zoom_reload > 0:
			self.zoom_reload -= dt
		else:
			self.color = (1, 0, 1, 1)

			
		if self.jumping:
			self.vy = 30
			self.jump_timer -= dt
		if self.jump_timer <= 0:
			self.jumping = False
			self.jump_timer = 0
			
		if not self.colliding and not self.jumping and not self.zoom:
			self.vy -= 8
		elif not self.jumping:
			self.vy = 0	
	
		self.vx *= self.fric
		
		self.x += self.vx
		self.y += self.vy
		
	def collision(self, obj, world):
		global g
		if obj == g.death:
			print "Dead."
			self.dead = True
		if obj != None and not world:
			self.color = (1, 0, 0, 1)
			self.jumping = True
			self.jump_timer = 0.1
			self.vx = -20
		super(Player, self).collision(obj, world)
			
class Game(pyglet.window.Window):
	def __init__(self):
		super(Game, self).__init__(width=800, height=600)
		self.init_gl()
		self.MENU = (self.update_menu, self.draw_menu)
		self.GAME = (self.update_game, self.draw_game)
		self.GAMEOVER = (self.update_gameover, self.draw_gameover)
		self.state = self.MENU
		pyglet.clock.schedule_interval(self.update, 1.0/30)
		pyglet.clock.set_fps_limit(30)
		self.keys = pyglet.window.key.KeyStateHandler()
		self.push_handlers(self.keys)
		random.seed(1337)

		self.hiscore_list = {}
		self.text_entry = "ENTER NAME"
		
		self.audio = pyglet.media.Player()
		self.audio.eos_action = "loop"
		self.setup()
		
	def setup(self):
		self.player = Player()
		self.push_handlers(self.player.keys)
		self.death = FireWall()
		self.objects = [self.player, self.death]
		self.world = []
		self.tiles = self.width * 2 / 40
		for x in range(self.tiles):
			self.world.append(GameObject(x*40, 0, None, 40, 40, color=(1, 0, 1, 0.5)))
		self.gravity = 8
		self.camera = GameObject(0, 0, None, self.width, self.height)
		self.camera.fric = 0
		self.shake = False
		self.rotate = False
		self.rotate_angle = 0
		self.rotate_speed = 2
		self.distance = 0
		self.total_time = 0
		self.audio.queue(pyglet.resource.media("intro.mp3"))
		self.audio.queue(pyglet.resource.media("game.mp3"))
		self.audio.play()
		
	def init_gl(self):
		# enable transparency
		glEnable(GL_BLEND)
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)

	def on_draw(self):
		self.state[1]()
		
	def update(self, dt):
		self.state[0](dt)
		
	def shake_screen(self, amount):
		glTranslatef(random.randint(-amount, amount), random.randint(-amount, amount), 0)
		
	def rotate_screen(self):
		if self.rotate and self.rotate_angle < 25:
			self.rotate_angle += self.rotate_speed
			self.shake_screen(40)
		elif not self.rotate and self.rotate_angle > 0:
			self.rotate_angle -= self.rotate_speed
			self.shake_screen(40)
		glTranslatef(self.camera.x, 0, 0)
		glRotatef(self.rotate_angle, 0, 0, 1)
		glTranslatef(-self.camera.x, 0, 0)
	
	# Game state	
	def draw_game(self):
		self.clear()
		glLoadIdentity()
		glTranslatef(-self.camera.x, -self.camera.y, 0)
		self.shake_screen(1)
		self.rotate_screen()
		for w in self.world:
			if self.camera.intersects(w):
				w.draw()
		for o in self.objects: o.draw()
		#print pyglet.clock.get_fps()
		
	def update_game(self, dt):
		if self.player not in self.objects or self.player.y + self.player.height < 0:
			self.player.dead = True
		
		if self.player.dead:
			self.distance = self.player.x
			self.state = self.GAMEOVER
			pyglet.resource.media("death.wav").play()
			self.audio.next()
			pyglet.resource.media("gameover.mp3").play()
			return
		
		# add new tiles
		x = self.world[-1].x
		while len(self.world) < self.tiles:
			x += 40
			if random.randrange(0, 50) == 35:
				x += 80
				continue
			self.world.append(GameObject(x, 0, None, 40, 40, color=(1, 0, 1, 0.5)))
		
		self.total_time += dt
		
		# random events
		if self.total_time > 4 and random.randint(0, 30) == 7:
			self.objects.append(Comet(self.camera.x, self.camera.y))
		if self.total_time > 20:
			self.rotate = True
		if self.total_time > 30:
			self.rotate = False
			if random.randint(0, 30) == 7:
				self.objects.append(Comet(self.camera.x, self.camera.y))
		if self.rotate and self.total_time > 22 and random.randrange(0, 70) == 3:
			self.objects.append(GameObject(
				self.camera.x + self.camera.width + 200, 100,
				None, 40, 40, color=(0, 1, 0, 1), vx=-4, vy=-8, physical=True
			))

		for o in self.objects:
			if self.rotate:
				self.player.vx -= 0.05
			self.check_collision(o)
			o.move(dt)
		if self.player.x >= self.camera.x + self.camera.width / 2 - self.player.width:
			self.camera.vx = self.player.vx
		self.camera.move(dt)
		# get rid of objects outside of screen
		self.clean_up()
		
	def check_collision(self, obj):
		objs = []
		objs.extend(self.world)
		objs.extend(self.objects)
		collision = False
		for w in objs:
			if w == obj:
				continue
			if obj.intersects(w):
				obj.collision(w, w in self.world)
				collision = True
		if collision == False:
			obj.collision(None, False)
		
	# menu state
	def update_menu(self, dt):
		if self.keys[key.ENTER]:
			self.state = (self.update_game, self.draw_game)
			self.audio.next()
			return
		return

	def draw_menu(self):
		self.clear()
		label = pyglet.text.Label("Press enter and start running..", font_name="Courier",
					  font_size=32, x=self.width/2, y=self.height/2,
					  anchor_x="center", anchor_y="center")
		label2 = pyglet.text.Label("Arrow keys to move, z to zoom..", font_name="Courier",
					  font_size=20, x=self.width/2, y=self.height/2 - 100,
					  anchor_x="center", anchor_y="center")
		label.draw()
		label2.draw()
		
	# game over state
	def update_gameover(self, dt):
		self.camera.x = 0
		self.camera.y = 0
		self.camera.vx = 0
		self.camera.vy = 0
		self.death.vx = 20
		for o in self.objects: o.move(1)
		remove = []
		for w in self.world:
			if w.intersects(self.death):
				remove.append(w)
		for r in remove: self.world.remove(r)
		remove = []
		for o in self.objects:
			if o == self.death:
				continue
			if o.intersects(self.death):
				remove.append(o)
		for r in remove: self.objects.remove(r)
		self.rotate = False
		if self.keys[key.ENTER]:
			if not self.text_entry == "ENTER NAME":
				self.keys[key.ENTER] = False
				self.text_entry = self.text_entry.strip()
				self.hiscore_list[int(self.distance)] = self.text_entry
				self.state = self.MENU
				self.audio.next()
				self.setup()
		self.clean_up()
	
	def draw_gameover(self):
		self.clear()
		glLoadIdentity()
		self.rotate_screen()
		for o in self.objects: o.draw()
		if random.randrange(0, 10) == 7:
			self.objects.append(Comet(self.camera.x, self.camera.y))
		label = pyglet.text.Label("Game Over..", font_name="Arial", font_size=62,
					  x = self.width/2, y = self.height/2,
					  anchor_x='center', anchor_y='center')
		label2 = pyglet.text.Label("%s made it %i pixels." % (self.text_entry, self.distance),
					   font_name="Arial", font_size=20,
					   x = self.width/2, y = self.height/2 - 100,
					   anchor_x="center", anchor_y="center")
		
		s = "Hiscores:\n"
		items = self.hiscore_list.items()
		items.sort(reverse=True)
		for k, v in items: s += v + " - " + str(k) + "\n"
		label3 = pyglet.text.Label(s, font_name="Arial", font_size=12,
					    x = self.width/2, y = self.height/2 - 200,
					   anchor_x="center", anchor_y="center", multiline=True,
					   width=200)
		label.draw()
		label2.draw()
		label3.draw()
		
	def on_text(self, text):
		if self.state != self.GAMEOVER:
			return
		else:
			if self.text_entry == "ENTER NAME":
				self.text_entry = ""
			self.text_entry += text

	def on_text_motion(self, motion):
		if motion == pyglet.window.key.MOTION_BACKSPACE:
			self.text_entry = self.text_entry[:-1]

	def clean_up(self):
		for i in range(len(self.objects)-2):
			o = self.objects[i]
			if o == self.player or o == self.death:
				continue
			if o.y + o.height < self.camera.y or o.x + o.width < self.camera.x:
				del self.objects[i]
		for i in range(len(self.world)-1):
			o = self.world[i]
			if o.y + o.height < self.camera.y or o.x + o.width < self.camera.x:
				del self.world[i]
	
import cPickle

g = Game()

f = pyglet.resource.file("s.rrr", "r")
for text in f.readlines():
	if text == "\n": continue
	k, v = tuple(text.split(","))
	g.hiscore_list[int(k)] = v.strip()
f.close()

pyglet.app.run()

f = pyglet.resource.file("s.rrr", "w")
for k, v in g.hiscore_list.iteritems():
	if v == "": v = "Unnamed"
	f.writelines(str(k) + "," + v + "\n")
f.close()
