"""
    Run! a game created for Pyweek
    Copyright (C) <2010> Thomas Berglund

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from GameObject import *
import random

class Particle(GameObject):
	def __init__(self, x, y, image, width, height, color=(1, 1, 1, 0), ttl=4, fade=False, physical=False):
		super(Particle, self).__init__(x, y, image, width, height, color)
		self.dead = False
		self.ttl = ttl
		self.fade = fade
		self.time = 0
		self.physical = physical
		#	pyglet.clock.schedule_interval(self.fade, 1)

	def move(self, dt):
		if self.time == self.ttl:
			self.dead = True
			return
		if self.fade:
			self.time += dt
			r, g, b, a = self.color
			a = 1 - (self.time * (1.0/self.ttl))
			self.color = (r, g, b, a)

class Emitter(GameObject):
	
	def __init__(self, x, y, p_x, p_y, particle_image, emitter_image, relative=False,
				 width=0, height=0, color=(0, 0, 0, 0), vx=0, vy=0, 
				 min_ttl=1, max_ttl=2, fade=False):
				
		super(Emitter, self).__init__(x, y, emitter_image, width, height, color, vx, vy)
		self.particle_image, self.emitter_image = None, None
		self.particle_image = particle_image
		self.emitter_image = emitter_image		
		self.relative = relative
		# if relative this is the amount of randomization from the emitters position
		# else, it's the range in which we randomize the position
		self.p_x = p_x
		self.p_y = p_y
		
		self.max_particles = 20
		self.particles = []
		
	def create_particle(self, width, height, min_ttl, max_ttl, fade=True, physical=True):
		if self.relative:
			x = random.randrange(self.x-self.p_x[0], self.x+self.p_x[1])
			y = random.randrange(self.y-self.p_y[0], self.y+self.p_y[1])
		else:
			x = random.randrange(self.x+self.p_x[0], self.x+self.p_x[1])
			y = random.randrange(self.y+self.p_y[0], self.y+self.p_y[1])
		ttl = random.randrange(min_ttl, max_ttl)
		self.particles.append(
			Particle(x, y, self.particle_image, width, height, ttl=ttl, fade=fade)
		)
		
	def draw(self):
		for p in self.particles:
			p.draw()
		super(Emitter, self).draw()
			
	def move(self, dt):
		for p in self.particles:
			p.move(dt)
			
		self.x += self.vx
		self.y += self.vy
			
		remove = []
		for p in self.particles:
			if p.dead: remove.append(p)
		for r in remove:
			self.particles.remove(r)